#!/usr/bin/perl -w

# Really useful when script is run from the relative path
BEGIN { unshift @INC, $1 if $0 =~ /(.*)[\/]/; }

use strict;
use klinstall;
use appdata;
use File::Path;

############################## text strings ##################################
my $UNINSTALL_MSG_1 = <<UNINSTALL_MSG_1_END;
%1 is not installed or has already been uninstalled.
UNINSTALL_MSG_1_END

my $UNINSTALL_MSG_2 = <<UNINSTALL_MSG_2_END;
%1 is required for %2 to work!
UNINSTALL_MSG_2_END

my $UNINSTALL_MSG_3 = <<UNINSTALL_MSG_3_END;
This script should not be run by the user - use your package manager to uninstall the software.
UNINSTALL_MSG_3_END

my $UNINSTALL_MSG_4 = <<UNINSTALL_MSG_4_END;
installation database is damaged, uninstall will be incompleted.
UNINSTALL_MSG_4_END

my $UNINSTALL_MSG_5 = <<UNINSTALL_MSG_5_END;
%1 has been uninstalled successfully.
UNINSTALL_MSG_5_END


############################### main() ######################################
	klinstall::check_for_root ("uninstall script");

	my ($logic, $pkgmgr) = klinstall::get_work_logic($0, \@ARGV);

	# parse cmdline
	my $upgrade = 0;
	foreach my $opt (@ARGV) {
	    if ($opt =~ /^--upgrade(=(\d))?$/) {
	        $upgrade = !defined($2) || ($2 ne '0') ? 1 : 0;
	    }
	}

	# Get application info
	my %APPINFO = appdata::get_application_info();

	# get app info
	my %INSTINFO = klinstall::read_app_info ($APPINFO{"ID"});
	if ( !%INSTINFO )
	{
		# for .tar.gz - abort deinstallation
		klinstall::fatal ($UNINSTALL_MSG_1, $APPINFO{"NAME"}) if !$pkgmgr;

		# for rpm, etc - generate a warning and continue uninstall
		klinstall::warning ($UNINSTALL_MSG_4);
		exit 0;
	}

	# wrong package manager?
	klinstall::fatal ($UNINSTALL_MSG_3) if $INSTINFO{"USED_PKGMGR"} && !$pkgmgr;

	# Check for dependend components
	if ( $INSTINFO{"DEPENDON"} )
	{
		foreach (split (/\s+/, $INSTINFO{"DEPENDON"}))
		{
			my %app = klinstall::read_app_info ($_);
			klinstall::fatal ($UNINSTALL_MSG_2, $APPINFO{"NAME"}, $app{"NAME"}) if !%app;
		}
	}

	# Custom pre-uninstall checks
	my $text = appdata::app_checks_preuninstall();
	klinstall::fatal ($text) if $text;

	# Application-specific uninstall
	appdata::app_unregister($upgrade);

	# removing does not required on upgrade
	exit 0 if $upgrade;

	# remove files
	if ( $INSTINFO{"FILE"} )
	{
		if ( ref $INSTINFO{"FILE"} ne "ARRAY" )
		{
			unlink $INSTINFO{"FILE"} or klinstall::warning ("Could not unlink file $INSTINFO{FILE}: $!");
		}
		else
		{
			foreach my $file (@{$INSTINFO{"FILE"}})
			{
				$file = "$INSTINFO{$1}/$2" if $file =~ /^(\w+)\/(.*)$/ && defined $INSTINFO{$1};
				next if !-f $file;
				unlink $file or klinstall::warning ("Could not unlink file $file: $!");
			}
		}
	}

    if ( $APPINFO{"SBINPATH"} )
    {
        my $markpath = $INSTINFO{"DATAROOT"};
        my $eula_accepted = "$markpath/eula_accepted";
        if ( -d $markpath && -e $eula_accepted ) {
            unlink $eula_accepted or klinstall::warning ("Could not remove file $eula_accepted: $!");
        }
    }

	# remove dirs in reversed order
	if ( $INSTINFO{"DIR"} )
	{
		foreach my $dir (reverse @{$INSTINFO{"DIR"}})
		{
			$dir = "$INSTINFO{$1}/$2" if $dir =~ /^(\w+)\/(.*)$/ && defined $INSTINFO{$1};
			next if !-d $dir;
			rmdir ($dir) or klinstall::warning ("Could not remove dir $dir: $!");
		}
	}
	
if ( $INSTINFO{"CFGPATH"} )
	{
		my $cfgdir = $INSTINFO{"CFGPATH"};
		chomp $cfgdir;

		if ($cfgdir ne "" and -e $cfgdir and -d $cfgdir)
		{
			opendir (DIR, $cfgdir) or klinstall::warning ("Could not open directory $cfgdir: $!");
			my @dir_items = readdir(DIR);
			my $isCfgDirNotEmpty = 0;
			
			foreach my $cur_item (@dir_items)
			{
				my $item_full_path = "$cfgdir/$cur_item";

				# delete any file found in the cfg dir
				if (-f $item_full_path)
				{

					unlink $item_full_path or klinstall::warning ("Could not remove file $item_full_path: $!");
				}

    			if (-d $item_full_path)
				{
					# skip . and .. pseudo folders
					next if ($cur_item =~ m/\.$/);

					if ($cur_item =~ m/connectors(64)?\.d/)
					{
						# do not remove connectors.d folder only if is is empty
						if (appdata::is_folder_not_empty($item_full_path))
						{
							$isCfgDirNotEmpty = 1;
							next;
						}
					}

					rmtree($item_full_path) or klinstall::warning ("Could not remove directory $item_full_path: $!");
				}
			}

			closedir(DIR);
			
			# delete cfg dir only if it is empty
			if (!$isCfgDirNotEmpty)
			{
				rmtree($cfgdir) or klinstall::warning ("Could not remove directory $cfgdir: $!")
			}
		}
		else
		{
			klinstall::warning ("Directory $cfgdir do not exists: $!")
		}
	}

	# remove the information from the storage
	klinstall::remove_app_info ($APPINFO{"ID"});
	
	klinstall::output ($UNINSTALL_MSG_5, $APPINFO{"NAME"}) if !$pkgmgr;
	exit 0;
